<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://uriahsvictor.com
 * @since      1.0.0
 *
 * @package    Lpac_DPS
 */
/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Lpac_DPS
 * @subpackage Lpac_DPS/includes
 * @author_name    Uriahs Victor <info@soaringleads.com>
 */
namespace Lpac_DPS\Bootstrap;

if ( !defined( 'ABSPATH' ) ) {
    exit;
}
use  Lpac_DPS\Bootstrap\Loader ;
use  Lpac_DPS\Bootstrap\I18n ;
use  Lpac_DPS\Bootstrap\Admin_Enqueues ;
use  Lpac_DPS\Bootstrap\Frontend_Enqueues ;
use  Lpac_DPS\Bootstrap\Setup_Cron ;
use  Lpac_DPS\Views\Admin\Settings_Panel\Render_Settings_Panel ;
use  Lpac_DPS\Views\Frontend\OrderDetails as OrderDetailsView ;
use  Lpac_DPS\Views\Admin\Order as Admin_Order_View ;
use  Lpac_DPS\Controllers\Checkout_Page\Order_Data as OrderDataController ;
use  Lpac_DPS\Controllers\Checkout_Page\Validate as Checkout_Page_Validation ;
use  Lpac_DPS\Controllers\Checkout_Page\Ajax\Handlers as Ajax_Handlers ;
use  Lpac_DPS\Controllers\Checkout_Page\ShippingMethods as ShippingMethodsController ;
use  Lpac_DPS\Controllers\CSF\Overrides as CSF_Overrides ;
use  Lpac_DPS\Controllers\Emails as EmailsController ;
use  Lpac_DPS\Notices\Loader as Notices_Loader ;
use  Lpac_DPS\Models\Base_Model ;
use  Lpac_DPS\Models\Plugin_Settings\General_Settings ;
use  Lpac_DPS\Notices\Notice ;
use  Lpac_DPS\Views\Admin\Admin as Admin_View ;
/**
 * Class Main.
 *
 * Class responsible for firing public and admin hooks.
 */
class Main
{
    /**
     * The loader that's responsible for maintaining and registering all hooks that power
     * the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      Lpac_DPS_Loader    $loader    Maintains and registers all hooks for the plugin.
     */
    protected  $loader ;
    /**
     * The unique identifier of this plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string    $plugin_name    The string used to uniquely identify this plugin.
     */
    protected  $plugin_name ;
    /**
     * The current version of the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string    $version    The current version of the plugin.
     */
    protected  $version ;
    /**
     * Plugin instance
     *
     * @var mixed
     */
    private static  $instance ;
    /**
     * Gets an instance of our plugin.
     *
     * @return Main()
     */
    public static function get_instance()
    {
        if ( null === self::$instance ) {
            self::$instance = new self();
        }
        return self::$instance;
    }
    
    /**
     * Define the core functionality of the plugin.
     *
     * Set the plugin name and the plugin version that can be used throughout the plugin.
     * Load the dependencies, define the locale, and set the hooks for the admin area and
     * the public-facing side of the site.
     *
     * @since    1.0.0
     */
    private function __construct()
    {
        $this->version = LPAC_DPS_VERSION;
        $this->plugin_name = LPAC_DPS_PLUGIN_NAME;
        $this->load_dependencies();
        $this->set_locale();
        $this->define_admin_hooks();
        $this->define_public_hooks();
    }
    
    /**
     * Load the required dependencies for this plugin.
     *
     * Create an instance of the loader which will be used to register the hooks
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function load_dependencies()
    {
        $this->loader = new Loader();
        new Render_Settings_Panel();
    }
    
    /**
     * Define the locale for this plugin for internationalization.
     *
     * Uses the Lpac_DPS_i18n class in order to set the domain and to register the hook
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function set_locale()
    {
        $plugin_i18n = new I18n();
        $this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );
    }
    
    /**
     * Register all of the hooks related to the admin area functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_admin_hooks()
    {
        $plugin_admin = new Admin_Enqueues();
        $admin_order_view = new Admin_Order_View();
        $plugin_admin_view = new Admin_View();
        $controller_csf_overrides = new CSF_Overrides();
        $bootstrap_cron_setup = new Setup_Cron();
        $notices_loader = new Notices_Loader();
        $notice = new Notice();
        $this->loader->add_action( 'admin_menu', $this, 'create_admin_menu' );
        $this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
        $this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
        $this->loader->add_filter(
            'plugin_action_links',
            $this,
            'add_plugin_action_links',
            PHP_INT_MAX,
            2
        );
        // Custom admin order columns
        $this->loader->add_filter( 'manage_edit-shop_order_columns', $plugin_admin_view, 'addDatetimeAdminListColumn' );
        $this->loader->add_action( 'manage_shop_order_posts_custom_column', $plugin_admin_view, 'addDatetimeAdminListColumnContent' );
        // Notices.
        $this->loader->add_action( 'admin_notices', $notices_loader, 'load_notices' );
        // Notices Ajax dismiss method (uncomment if making use of notice class)
        $this->loader->add_action( 'wp_ajax_lpac_dps_dismiss_notice', $notice, 'dismiss_notice' );
        // Metaboxes.
        $this->loader->add_action( 'add_meta_boxes', $admin_order_view, 'create_metabox' );
        // Codestar Framework overrides.
        $csf_id = LPAC_DPS_CSF_ID;
        $this->loader->add_filter(
            "csf_{$csf_id}_save",
            $controller_csf_overrides,
            'handle_order_type_switchers',
            10,
            2
        );
        // Cron tasks.
        $this->loader->add_action( 'admin_init', $bootstrap_cron_setup, 'set_cron_tasks' );
    }
    
    /**
     * Register all of the hooks related to the public-facing functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_public_hooks()
    {
        $plugin_public = new Frontend_Enqueues();
        $controller_checkout_order_data = new OrderDataController();
        $controller_checkout_page_validation = new Checkout_Page_Validation();
        $controller_ajax_handlers = new Ajax_Handlers();
        $controller_shipping_methods = new ShippingMethodsController();
        $controller_emails = new EmailsController();
        $view_order_details_page = new OrderDetailsView();
        $this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
        $this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );
        /**
         * Set WC session data for plugin.
         */
        $this->loader->add_action( 'woocommerce_checkout_update_order_review', $controller_shipping_methods, 'setupSessionData' );
        /**
         * Output our fields on the checkout page.
         */
        add_action( 'plugins_loaded', function () {
            $output_location = Base_Model::get_setting( 'general__fields_display_location', 'woocommerce_review_order_before_payment' );
            $output_location = apply_filters( 'dps_fields_checkout_page_location', $output_location );
            $class = new \Lpac_DPS\Views\Frontend\Checkout_Page();
            $priority = apply_filters( 'dps_fields_display_location_filter_priority', 10 );
            add_action( $output_location, array( $class, 'add_checkout_fields' ), $priority );
        } );
        /**
         * Ajax Handler for getting available delivery times when a day of the week is selected.
         */
        $this->loader->add_action( 'wp_ajax_lpac_dps_get_times', $controller_ajax_handlers, 'get_times_ajax_handler' );
        $this->loader->add_action( 'wp_ajax_nopriv_lpac_dps_get_times', $controller_ajax_handlers, 'get_times_ajax_handler' );
        /**
         * Validate our checkout fields.
         */
        $delivery_date_enabled = Base_Model::get_setting( 'delivery__enable_date_feature' );
        $pickup_date_enabled = Base_Model::get_setting( 'pickup__enable_date_feature' );
        
        if ( !empty($delivery_date_enabled) && !empty($pickup_date_enabled) ) {
            $this->loader->add_action(
                'woocommerce_after_checkout_validation',
                $controller_checkout_page_validation,
                'validate_future_date',
                10,
                2
            );
            $this->loader->add_action(
                'woocommerce_after_checkout_validation',
                $controller_checkout_page_validation,
                'validate_date_field',
                10,
                2
            );
            $this->loader->add_action(
                'woocommerce_after_checkout_validation',
                $controller_checkout_page_validation,
                'validate_time_field',
                10,
                2
            );
        }
        
        /**
         * Save delivery/pickup date and time
         */
        $this->loader->add_action( 'woocommerce_checkout_update_order_meta', $controller_checkout_order_data, 'save_dps_data' );
        /*
         * Output LPAC DPS details on thank you and view order pages
         */
        $this->loader->add_action(
            'woocommerce_order_details_after_order_table',
            $view_order_details_page,
            'outputDeliveryPickupDetails',
            9
        );
        /**
         * Show respective shipping methods based on the order type selected.
         */
        $this->loader->add_filter(
            'woocommerce_package_rates',
            $controller_shipping_methods,
            'alterShippingMethods',
            PHP_INT_MAX,
            2
        );
        /*
         * Display date and time in order emails.
         */
        $enable_datetime_in_emails = General_Settings::getIncludeDateTimeInEmailsSetting();
        
        if ( $enable_datetime_in_emails ) {
            $datetime_location = General_Settings::getDateTimeInEmailsLocation();
            $this->loader->add_action(
                $datetime_location,
                $controller_emails,
                'addDateTimeToEmails',
                20,
                4
            );
        }
    
    }
    
    /**
     * Run the loader to execute all of the hooks with WordPress.
     *
     * @since    1.0.0
     */
    public function run()
    {
        $this->loader->run();
    }
    
    /**
     * The name of the plugin used to uniquely identify it within the context of
     * WordPress and to define internationalization functionality.
     *
     * @since     1.0.0
     * @return    string    The name of the plugin.
     */
    public function get_plugin_name()
    {
        return $this->plugin_name;
    }
    
    /**
     * The reference to the class that orchestrates the hooks with the plugin.
     *
     * @since     1.0.0
     * @return    Lpac_DPS_Loader    Orchestrates the hooks of the plugin.
     */
    public function get_loader()
    {
        return $this->loader;
    }
    
    /**
     * Retrieve the version number of the plugin.
     *
     * @since     1.0.0
     * @return    string    The version number of the plugin.
     */
    public function get_version()
    {
        return $this->version;
    }
    
    /**
     * Add action Links for plugin.
     *
     * @param array  $plugin_actions Plugin actions.
     * @param string $plugin_file Plugin file name.
     * @return array
     */
    public function add_plugin_action_links( $plugin_actions, $plugin_file )
    {
        return $plugin_actions;
    }
    
    /**
     * Create our SoaringLeads menu item.
     *
     * @return void
     * @since 1.0.3
     */
    public function create_admin_menu() : void
    {
        $icon = file_get_contents( LPAC_DPS_PLUGIN_ASSETS_DIR . 'admin/img/menu-icon.svg' );
        $icon = 'data:image/svg+xml;base64,' . base64_encode( $icon );
        $main_menu = menu_page_url( 'sl-plugins-menu', false );
        
        if ( !empty($main_menu) ) {
            return;
            // Menu already added by another SoarngLeads plugin
        }
        
        add_menu_page(
            __( 'SoaringLeads Plugins', 'delivery-and-pickup-scheduling-for-woocommerce' ),
            'SoaringLeads',
            'manage_options',
            'sl-plugins-menu',
            array( $this, 'output_root_submenu_upsells' ),
            $icon,
            '57.10'
        );
    }
    
    /**
     * HTML for root SoaringLeads page.
     *
     * Populate with upsell content.
     *
     * @since 1.0.3
     */
    public function output_root_submenu_upsells()
    {
        ?>
		<h1><?php 
        esc_html_e( 'Check out our available plugins', 'delivery-and-pickup-scheduling-for-woocommerce' );
        ?></h1>
		<hr style='margin-bottom: 40px'/>
		
		<div style='margin-bottom: 40px'>
		<a href='https://lpacwp.com/?utm_source=wpadmin&utm_medium=sl-plugins-page&utm_campaign=plugins-upsell' target='_blank'><img src='<?php 
        echo  esc_attr( LPAC_DPS_PLUGIN_ASSETS_PATH_URL . 'admin/img/lpac.png' ) ;
        ?>' /></a>
		<p style='font-size: 18px; font-weight: 700;'><?php 
        esc_html_e( 'Let customers choose their shipping or pickup location using a map during checkout.', 'delivery-and-pickup-scheduling-for-woocommerce' );
        ?></p>	
		<a href='https://lpacwp.com/?utm_source=wpadmin&utm_medium=sl-plugins-page&utm_campaign=plugins-upsell' target='_blank' class='button-primary'><?php 
        esc_html_e( 'Learn More', 'delivery-and-pickup-scheduling-for-woocommerce' );
        ?></a>
		</div>
		
		<div style='margin-bottom: 40px'>
		<a href='https://dpswp.com/?utm_source=wpadmin&utm_medium=sl-plugins-page&utm_campaign=plugins-upsell' target='_blank'><img src='<?php 
        echo  esc_attr( LPAC_DPS_PLUGIN_ASSETS_PATH_URL . 'admin/img/delivery-and-pickup-scheduling.png' ) ;
        ?>' /></a>
		<p style='font-size: 18px; font-weight: 700;'><?php 
        echo  esc_html( 'Allow customers to set their delivery/pickup date and time during order checkout.', 'delivery-and-pickup-scheduling-for-woocommerce' ) ;
        ?></p>	
		<a href='https://dpswp.com/?utm_source=wpadmin&utm_medium=sl-plugins-page&utm_campaign=plugins-upsell' target='_blank' class='button-primary'><?php 
        esc_html_e( 'Learn More', 'delivery-and-pickup-scheduling-for-woocommerce' );
        ?></a>
		</div>
		
		<?php 
    }

}