<?php
/**
 * Orchestrate Checkout Page related logic.
 *
 * Author:          Uriahs Victor
 * Created on:      18/10/2022 (d/m/y)
 *
 * @link    https://uriahsvictor.com
 * @since   1.0.0
 * @package Controllers
 */

namespace Lpac_DPS\Controllers\Checkout_Page;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

use Lpac_DPS\Models\Plugin_Settings\General_Settings;
use Lpac_DPS\Models\Plugin_Settings\Order_Type as Order_Type_Settings;
use Lpac_DPS\Models\Plugin_Settings\Scheduling as Scheduling_Settings;
use Lpac_DPS\Models\Plugin_Settings\Localization as Localization_Settings;

/**
 * Class responsible for setting up frontend fields.
 *
 * @package Lpac_DPS\Controllers\Checkout_Page
 * @since 1.0.0
 */
class Setup_Fields extends Base_Checkout_Page_Controller {

	/**
	 * Bring our calendar settings together for usage in frontend.
	 *
	 * @return string
	 * @since 1.0.0
	 */
	public function get_calendar_settings() : string {

		$delivery_settings = new Scheduling_Settings( 'delivery' );
		$pickup_settings   = new Scheduling_Settings( 'pickup' );

		$general_settings      = new General_Settings();
		$order_type_settings   = new Order_Type_Settings();
		$localization_settings = new Localization_Settings();

		// Delivery.
		$available_delivery_days         = $delivery_settings->get_available_days();
		$delivery_minimum_days_in_future = $delivery_settings->get_minimum_days_in_future();
		$delivery_maximum_days_in_future = $delivery_settings->get_maximum_days_in_future();
		$delivery_off_days               = $delivery_settings->get_off_days();
		$delivery_maxed_dates            = $delivery_settings->get_maxed_dates();

		// Pickup.
		$available_pickup_days         = $pickup_settings->get_available_days();
		$pickup_minimum_days_in_future = $pickup_settings->get_minimum_days_in_future();
		$pickup_maximum_days_in_future = $pickup_settings->get_maximum_days_in_future();
		$pickup_off_days               = $pickup_settings->get_off_days();
		$pickup_maxed_dates            = $pickup_settings->get_maxed_dates();

		// Locale.
		$weekdays_shorthand = $localization_settings->get_weekdays( 'shorthand' );
		$weekdays_longhand  = $localization_settings->get_weekdays( 'longhand' );
		$months_shorthand   = $localization_settings->get_months( 'shorthand' );
		$months_longhand    = $localization_settings->get_months( 'longhand' );

		// Misc.
		$first_day_of_week = $general_settings->get_first_day_of_week();

		$date_format         = $general_settings::get_preferred_date_format();
		$general_time_format = $general_settings::get_preferred_time_format();
		$default_order_type  = $order_type_settings->get_default_order_type();

		$order_type = array(
			'default'  => $default_order_type,
			'delivery' => Localization_Settings::getCheckoutDeliveryText(),
			'pickup'   => Localization_Settings::getCheckoutPickupText(),
		);

		$calendar_config = array(
			'delivery'       => array(
				'availableDays'   => $available_delivery_days,
				'minDaysInFuture' => $delivery_minimum_days_in_future,
				'maxDaysInFuture' => $delivery_maximum_days_in_future,
				'offDays'         => $delivery_off_days,
				'maxedDates'      => $delivery_maxed_dates,
			),
			'pickup'         => array(
				'availableDays'   => $available_pickup_days,
				'minDaysInFuture' => $pickup_minimum_days_in_future,
				'maxDaysInFuture' => $pickup_maximum_days_in_future,
				'offDays'         => $pickup_off_days,
				'maxedDates'      => $pickup_maxed_dates,
			),
			'firstDayOfWeek' => $first_day_of_week,
			'dateFormat'     => $date_format,
			'locale'         => array(
				'weekdaysShorthand' => $weekdays_shorthand,
				'weekdaysLonghand'  => $weekdays_longhand,
				'monthsShorthand'   => $months_shorthand,
				'monthsLonghand'    => $months_longhand,
			),

		);

		$timepicker_config = array(
			'timeFormat' => $general_time_format,
		);

		$calendar_config   = wp_json_encode( $calendar_config );
		$timepicker_config = wp_json_encode( $timepicker_config );
		$order_type_config = wp_json_encode( $order_type );

		$globals = "
		var lpacDPSCalendarConfig = '$calendar_config';
		var lpacDPSTimePickerConfig = '$timepicker_config';
		var lpacDPSOrderTypeConfig = '$order_type_config';
		";

		return $globals;
	}

}
