<?php

/**
 * Handles checkout page view related logic.
 *
 * Author:          Uriahs Victor
 * Created on:      13/10/2022 (d/m/y)
 *
 * @link    https://uriahsvictor.com
 * @since   1.0.0
 * @package Views
 */
namespace Lpac_DPS\Views\Frontend;

if ( !defined( 'ABSPATH' ) ) {
    exit;
}
use  Lpac_DPS\Helpers\Functions ;
use  Lpac_DPS\Models\Plugin_Settings\Localization ;
use  Lpac_DPS\Views\Base_View ;
use  Lpac_DPS\Models\Plugin_Settings\Order_Type as Order_Type_Settings ;
use  Lpac_DPS\Models\Plugin_Settings\Scheduling as Scheduling_Settings ;
/**
 * Class Checkout_Page.
 */
class Checkout_Page extends Base_View
{
    /**
     * Default Order Type selected at checkout.
     *
     * @var string
     * @since 1.0.0
     */
    private  $default_order_type ;
    /**
     * Instance of order type settings.
     *
     * @var Order_Type_Settings
     * @since 1.0.0
     */
    private  $order_type_settings ;
    /**
     * Class constructor.
     *
     * @return void
     * @since 1.0.0
     */
    public function __construct()
    {
        parent::__construct();
        $this->order_type_settings = new Order_Type_Settings();
        $this->default_order_type = $this->order_type_settings->get_default_order_type();
    }
    
    /**
     * Create our Switcher element.
     *
     * @return void
     * @since 1.0.0
     */
    private function create_switcher() : void
    {
        // If neither delivery nor pickup scheduling is't enabled on the site then bail.
        if ( $this->order_type_settings->is_delivery_enabled() === false || $this->order_type_settings->is_pickup_enabled() === false ) {
            return;
        }
        $pickup_text = Localization::getCheckoutPickupText();
        $delivery_text = Localization::getCheckoutDeliveryText();
        $change_order_to_text = Localization::getCheckoutChangeOrderToText();
        $default_text = ( $this->delivery === $this->default_order_type ? $pickup_text : $delivery_text );
        ?>
			<div style='display: flex; align-items: center; margin-bottom: 20px'>
				<span style='margin-right: 10px'>
					<?php 
        echo  esc_html( $change_order_to_text ) ;
        ?> <strong><?php 
        echo  esc_html( $default_text ) ;
        ?></strong>
				</span> 
				<label class='lpac-dps-switch'>
					<input id='lpac-dps-order-type' type='checkbox' >
					<span class='lpac-dps-slider round'></span>
				</label>
			</div>
		<?php 
    }
    
    /**
     * Create delivery scheduling fields on checkout page.
     *
     * @return void
     * @since 1.0.0
     */
    private function create_delivery_fields( $field_class = array() ) : void
    {
        $delivery_enabled = $this->order_type_settings->is_delivery_enabled();
        if ( false === $delivery_enabled ) {
            return;
        }
        $delivery_settings = new Scheduling_Settings( $this->delivery );
        $delivery_date_enabled = $delivery_settings->is_date_enabled();
        $delivery_time_enabled = $delivery_settings->is_time_enabled();
        $label_delivery_date = $delivery_settings->get_date_selector_label();
        $delivery_date_required = $delivery_settings->is_date_required();
        $label_delivery_time = $delivery_settings->get_timefield_label();
        $delivery_required = $delivery_settings->is_time_required();
        $delivery_times = $delivery_settings->get_saved_timeslots();
        $delivery_customer_note = $delivery_settings->get_customer_note();
        $delivery_customer_font_size = $delivery_settings->get_customer_note_font_size();
        do_action( 'lpac_dps_before_delivery_fields', $delivery_settings );
        if ( $delivery_date_enabled ) {
            woocommerce_form_field( 'lpac_dps_delivery_date', array(
                'type'     => 'text',
                'label'    => $label_delivery_date,
                'required' => $delivery_date_required,
                'priority' => PHP_INT_MAX,
                'class'    => $field_class,
            ), '' );
        }
        if ( $delivery_time_enabled && !empty($delivery_times) ) {
            woocommerce_form_field( 'lpac_dps_delivery_time', array(
                'type'     => 'text',
                'label'    => $label_delivery_time,
                'required' => $delivery_required,
                'priority' => PHP_INT_MAX,
                'class'    => $field_class,
            ), '' );
        }
        
        if ( !empty($delivery_customer_note) ) {
            $class = ( $this->pickup === $this->default_order_type ? 'hidden' : '' );
            ?>
			<p id="delivery-customer-note" class="<?php 
            echo  esc_attr( $class ) ;
            ?>" style="font-style: italic; font-size: <?php 
            echo  esc_attr( $delivery_customer_font_size ) ;
            ?>"><?php 
            echo  esc_html( $delivery_customer_note ) ;
            ?></p>
			<?php 
        }
        
        do_action( 'lpac_dps_after_delivery_fields', $delivery_settings );
    }
    
    /**
     * Create pickup scheduling fields on checkout page.
     *
     * @return void
     * @since 1.0.0
     */
    private function create_pickup_fields( $field_class = array() ) : void
    {
        $pickup_enabled = $this->order_type_settings->is_pickup_enabled();
        if ( false === $pickup_enabled ) {
            return;
        }
        $pickup_settings = new Scheduling_Settings( $this->pickup );
        $pickup_date_enabled = $pickup_settings->is_date_enabled();
        $pickup_time_enabled = $pickup_settings->is_time_enabled();
        $label_pickup_date = $pickup_settings->get_date_selector_label();
        $pickup_date_required = $pickup_settings->is_date_required();
        $label_pickup_time = $pickup_settings->get_timefield_label();
        $pickup_required = $pickup_settings->is_time_required();
        $pickup_times = $pickup_settings->get_saved_timeslots();
        $pickup_customer_note = $pickup_settings->get_customer_note();
        $pickup_customer_font_size = $pickup_settings->get_customer_note_font_size();
        do_action( 'lpac_dps_before_pickup_fields', $pickup_settings );
        if ( $pickup_date_enabled ) {
            woocommerce_form_field( 'lpac_dps_pickup_date', array(
                'type'     => 'text',
                'label'    => $label_pickup_date,
                'required' => $pickup_date_required,
                'priority' => PHP_INT_MAX,
                'class'    => $field_class,
            ), '' );
        }
        if ( $pickup_time_enabled && !empty($pickup_times) ) {
            woocommerce_form_field( 'lpac_dps_pickup_time', array(
                'type'     => 'text',
                'label'    => $label_pickup_time,
                'required' => $pickup_required,
                'priority' => PHP_INT_MAX,
                'class'    => $field_class,
            ), '' );
        }
        
        if ( !empty($pickup_customer_note) ) {
            $class = ( $this->delivery === $this->default_order_type ? 'hidden' : '' );
            ?>
				<p id= "pickup-customer-note" class="<?php 
            echo  esc_attr( $class ) ;
            ?>" style="font-style: italic; font-size: <?php 
            echo  esc_html( $pickup_customer_font_size ) ;
            ?>"><?php 
            echo  esc_html( $pickup_customer_note ) ;
            ?></p>
			<?php 
        }
        
        do_action( 'lpac_dps_after_pickup_fields', $pickup_settings );
    }
    
    /**
     * Create our Delivery and Pickup input fields.
     *
     * @return void
     * @since 1.0.0
     */
    private function create_dps_fields() : void
    {
        do_action( 'lpac_dps_before_fields' );
        $pickup_field_class = ( $this->pickup === $this->default_order_type ? array() : array( 'hidden' ) );
        $delivery_field_class = ( $this->delivery === $this->default_order_type ? array() : array( 'hidden' ) );
        // Free version.
        woocommerce_form_field( 'lpac_dps_order_type', array(
            'type'              => 'text',
            'label'             => 'Order Type',
            'priority'          => PHP_INT_MAX,
            'required'          => true,
            'class'             => ( LPAC_DPS_DEBUG ? array() : array( 'hidden' ) ),
            'custom_attributes' => array(
            'readonly' => true,
        ),
        ), $this->default_order_type );
        $this->create_delivery_fields( $delivery_field_class );
        $this->create_pickup_fields( $pickup_field_class );
        do_action( 'lpac_dps_after_fields' );
    }
    
    /**
     * Create our custom checkout fields.
     *
     * @return void
     * @since 1.0.0
     */
    public function add_checkout_fields() : void
    {
        $plugin_enabled = $this->base_model::get_setting( 'general__enable_dps_plugin', true );
        if ( empty($plugin_enabled) ) {
            return;
        }
        // If both delivery and pickups are disabled on the site then don't show any fields.
        if ( $this->order_type_settings->is_delivery_enabled() === false && $this->order_type_settings->is_pickup_enabled() === false ) {
            return;
        }
        /**
         * Its very important that we have the expected value for the default order type or else our logic can break.
         * The default_order_type is being set as the default value for our order type field in create_dps_fields()
         * This value is later consumed in PHP $_POST logic to save the order type and run further logic when viewing the order.
         *
         * @see Lpac_DPS\Views\Admin\Order::create_metabox()
         */
        
        if ( $this->delivery !== $this->default_order_type && $this->pickup !== $this->default_order_type ) {
            return;
            // Something is wrong, bail.
        }
        
        // Section header on checkout page.
        ?>
			<h3>
				<?php 
        echo  esc_html( Localization::getCheckoutOrderTypeText() ) ;
        ?> 
				<span class='lpac-dps-current-order-type'>
					<?php 
        $delivery_text = Localization::getCheckoutDeliveryText();
        $pickup_text = Localization::getCheckoutPickupText();
        /**
         * Free doesn't have user role logic so just check which order type is default.
         */
        
        if ( $this->delivery === $this->default_order_type ) {
            echo  esc_html( $delivery_text ) ;
        } else {
            echo  esc_html( $pickup_text ) ;
        }
        
        ?>
				</span>
			</h3>
		<?php 
        $this->create_switcher();
        $this->create_dps_fields();
    }

}